#include "stdafx.h"
#include "ThreadProbeDll.h"
#include "ThreadFunctions.h"
#include "..\SimplestComPort\ComPort.h"

extern CRITICAL_SECTION CriticalSection;
void DisplayLastErrorMessage(void);

//
//  Variables
//
HANDLE WorkThread = NULL;
volatile BOOL ThreadStopFlag = FALSE;

//
//  The working thread loop function
//
DWORD WINAPI WorkThreadLoop(LPVOID param)
{
  ComPort_PresetParameters(19200,8,NOPARITY,ONESTOPBIT);
  BOOL com_opened = ComPort_Open("COM2");
  Serial_flush();
  while(!ThreadStopFlag) {
    if(com_opened) {
      int c = ComPort_GetByte(100);	// Wait for char for 100 msec
      if(c != -1) Serial_PutFifo(c);
    }
  }
  ComPort_Close();
  return 0;
}

#define SERIAL_FIFO_SIZE	256
#define SERIAL_FIFO_SIZE_MASK	0xFF

// SERIAL_FIFO_SIZE-1 chars MAX! head == tail -> empty
static volatile int SerialFifo[SERIAL_FIFO_SIZE];
static volatile int SerialFifoHead = 0;
static volatile int SerialFifoTail = 0;

void Serial_flush(void)
{
  EnterCriticalSection(&CriticalSection);
  SerialFifoHead = SerialFifoTail = 0;
  LeaveCriticalSection(&CriticalSection);
}

BOOL Serial_empty(void)
{
  BOOL empty;

  EnterCriticalSection(&CriticalSection);
// FIFO empty if pointers equal
  empty = SerialFifoTail == SerialFifoHead;
  LeaveCriticalSection(&CriticalSection);
  return empty;
}

int Serial_getc(void)
{
  int c;

  while(Serial_empty()) Sleep(10);	// Wait for char (and free processor)
  EnterCriticalSection(&CriticalSection);
  c = SerialFifo[SerialFifoHead];
// FIFO addr range = 0...(SERIAL_FIFO_SIZE-1)
  SerialFifoHead = (SerialFifoHead+1) & SERIAL_FIFO_SIZE_MASK;
  LeaveCriticalSection(&CriticalSection);
  return c;
}

void Serial_PutFifo(int c)
{
  int new_tail;

  EnterCriticalSection(&CriticalSection);
  new_tail = (SerialFifoTail+1) & SERIAL_FIFO_SIZE_MASK;
  // Check for FIFO has SERIAL_FIFO_SIZE chars - it is full, new chars are lost
  if(new_tail != SerialFifoHead) { 
    SerialFifo[SerialFifoTail] = c;
    SerialFifoTail = new_tail;		 // FIFO addr range = 0...(SERIAL_FIFO_SIZE-1)
  }
  LeaveCriticalSection(&CriticalSection);
}
